package com.tzld.piaoquan.recommend.server.service.recall.strategy;

import com.google.common.collect.Lists;
import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.filter.FilterParam;
import com.tzld.piaoquan.recommend.server.service.filter.FilterResult;
import com.tzld.piaoquan.recommend.server.service.filter.RegionFilterService;
import com.tzld.piaoquan.recommend.server.service.recall.FilterParamFactory;
import com.tzld.piaoquan.recommend.server.service.recall.RecallParam;
import com.tzld.piaoquan.recommend.server.service.recall.RecallStrategy;
import com.tzld.piaoquan.recommend.server.service.score.ScorerUtils;
import com.tzld.piaoquan.recommend.server.service.score4recall.ScorerPipeline4Recall;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * @author zhangbo
 */
@Component
public class AppFallbackRecallStrategy implements RecallStrategy {

    public static final String PUSH_FORM = "app_fallback_recall_strategy";

    @Autowired
    private RegionFilterService filterService;

    @Override
    public List<Video> recall(RecallParam param) {
        // 1 获取省份key 放入参数map中
        String regionCode = param.getRegionCode();
        if (StringUtils.isBlank(regionCode)) {
            regionCode = "";
        }
        Map<String, String> param4Model = new HashMap<>(1);
        param4Model.put("region_code", regionCode);
        // 2 通过model拿到召回list
        ScorerPipeline4Recall pipeline = ScorerUtils.getScorerPipeline4Recall("app_fallback_config.conf");
        List<List<Pair<Long, Double>>> results = pipeline.recall(param4Model);
        if (CollectionUtils.isEmpty(results)) {
            return Collections.emptyList();
        }
        List<Pair<Long, Double>> result = results.get(0);
        for (int i = 1; i < results.size(); ++i) {
            result.addAll(results.get(i));
        }
        Map<Long, Double> videoMap = new LinkedHashMap<>();
        for (Pair<Long, Double> v : result) {
            videoMap.put(v.getLeft(), v.getRight());
        }
        FilterParam filterParam = FilterParamFactory.create(param, Lists.newArrayList(videoMap.keySet()));
        filterParam.setForceTruncation(10000);
        filterParam.setConcurrent(true);
        filterParam.setNotUsePreView(false);
        FilterResult filterResult = filterService.filter(filterParam);
        List<Video> videosResult = new ArrayList<>();
        if (filterResult != null && CollectionUtils.isNotEmpty(filterResult.getVideoIds())) {
            filterResult.getVideoIds().stream().limit(500)
                    .forEach(vid -> {
                        Video video = new Video();
                        video.setVideoId(vid);
                        video.setAbCode(param.getAbCode());
                        video.setRovScore(videoMap.get(vid));
                        video.setPushFrom(pushFrom());
                        videosResult.add(video);
                    });
        }
        return videosResult;
    }

    @Override
    public String pushFrom() {
        return PUSH_FORM;
    }


}
